/**
 * 
 * $$Author: dominik $$
 * $$Date: 2006/06/12 14:12:39 $$
 * $$Revision: 1.47 $$
 * 
 */
package gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.vecmath.Matrix3f;

import membranModel.MembranModel;
import membranModel.MembranModelSingleton;
import protein.Protein;

public class JPreview extends JPanel {
	public Vector pdbFiles;

	private Protein protein;

	private Protein lipid;

	private JWorkPanel workPanel;

	// Create two viewer to switch between MembranePreview and ProteinPreview
	JPanel membraneJPanel;

	JPanel jPanelJmol;

	JPanel jPanelJmolButtons;

	// Viewer with lipids
	private JmolPdbPreview jmolPdbViewer;

	// Just the Jmol viewer to visualize the complete membrane
	private JmolPdbPreview jmolMemViewer;

	/**
	 * The Component Listener for JPreview. It is invoked each time the size of
	 * the PDBPreview window is changed.
	 */
	private class JPreviewResizedListener implements ComponentListener {

		private JPreview jPreview;

		// this double influences the quantity of the shown lipids in the
		// preview
		private double factorQuantity = 2.4;

		/**
		 * The constructor of the JPreviewResizedListener gets the JPreview in
		 * order to invoke a repaint on the whole JPreview-window.
		 * 
		 * @param pPreview
		 *            jPreview
		 */
		public JPreviewResizedListener(JPreview pPreview) {
			jPreview = pPreview;
		}

		/**
		 * The Listener for the resize of the PreviewWindow. It computes the
		 * size of the PDB-zoom and the lipid-number on both sides of the
		 * protein.
		 */
		public void componentResized(ComponentEvent arg0) {

			// the viewer is resized, if there is already a protein scaled in
			// the viewer
			if (jmolPdbViewer.getViewer().getScalePixelsPerAngstrom() > 0) {

				// decomment Sytem.outs to show infos
				// System.out.println("\n***resize-start***");

				jmolPdbViewer.getViewer().setScalePixelsPerAngstrom(
						jmolPdbViewer.getViewer().getRotationRadius() / 3);
				double scaleFactor = jmolPdbViewer.getViewer()
						.getScalePixelsPerAngstrom();
				double sizeFrame = jmolPdbViewer.getFrame().getWidth();
				double lipidNumber = 0;
				// computation of the lipid-quantity
				lipidNumber = (((sizeFrame * factorQuantity) / (scaleFactor * 2 * lipid
						.getLength())) - ((protein.getLength()) / (2 * lipid
						.getLength())));
				jmolPdbViewer.getViewer().setLipidNumber((int) lipidNumber);

				// System.out.println("LipidNumber: "+(int)lipidNumber);
				// System.out.println("jPFrameWidth:
				// "+jmolPdbViewer.getFrame().getWidth()+" jPParentWidth:
				// "+jmolPdbViewer.getParent().getWidth());
				// System.out.println("Diff:
				// "+(jmolPdbViewer.getParent().getWidth()-jmolPdbViewer.getFrame().getWidth())/2);
				// System.out.println("Scale des PDBs im Viewer:
				// "+jmolPdbViewer.getViewer().getScalePixelsPerAngstrom());
				// System.out.println("Verhltnis von Protein und Lipid: " +
				// protein.getLength() + "_:_" + lipid.getLength());

				// System.out.println("***resize-end***");
			}

			jPreview.getParent().repaint();

		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.event.ComponentListener#componentMoved(java.awt.event.ComponentEvent)
		 */
		public void componentMoved(ComponentEvent arg0) {
			// TODO Auto-generated method stub

		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.event.ComponentListener#componentShown(java.awt.event.ComponentEvent)
		 */
		public void componentShown(ComponentEvent arg0) {
			// TODO Auto-generated method stub

		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.event.ComponentListener#componentHidden(java.awt.event.ComponentEvent)
		 */
		public void componentHidden(ComponentEvent arg0) {
			// TODO Auto-generated method stub
			// jPreview.computePDBManipulation();
		}

	}

	/**
	 * The Status Listener for the LipidMembraneButton
	 */
	private class LipidButtonListener implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			if (getShowTransparentLipids()) {
				setShowTransparentLipids(false);
			} else {
				setShowTransparentLipids(true);
			}
			// System.out.println("Viewer:
			// "+jmolPdbViewer.getViewer().getModelSetProperties().getProperty("Cartoon").toString());
		}
	}

	/**
	 * The Status Listener for the PrimaryStructureButton -> JmolConstants.java
	 * for constants
	 */
	private class AxesButtonListener implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			if (jmolPdbViewer.getViewer().getShapeShow(16)) {
				jmolPdbViewer.getViewer().setShapeShow(16, false);
			} else {
				jmolPdbViewer.getViewer().setShapeShow(16, true);
			}
		}
	}

	/**
	 * The Status Listener for the SecondaryStructureButton ->
	 * JmolConstants.java for constants
	 */
	private class SecStructButtonListener implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			if (jmolPdbViewer.getViewer().getShapeShow(10)) {
				jmolPdbViewer.getViewer().setShapeShow(8, false);
				jmolPdbViewer.getViewer().setShapeShow(9, false);
				jmolPdbViewer.getViewer().setShapeShow(10, false);
				jmolPdbViewer.getViewer().setShapeShow(11, false);
				jmolPdbViewer.getViewer().setShapeShow(12, false);
				jmolPdbViewer.getViewer().setShapeShow(13, false);
				jmolPdbViewer.getViewer().setShapeShow(14, false);
				jmolPdbViewer.getViewer().setShapeShow(15, false);
			} else {
				jmolPdbViewer.getViewer().setShapeShow(8, true);
				jmolPdbViewer.getViewer().setShapeShow(9, true);
				jmolPdbViewer.getViewer().setShapeShow(10, true);
				jmolPdbViewer.getViewer().setShapeShow(11, true);
				jmolPdbViewer.getViewer().setShapeShow(12, true);
				jmolPdbViewer.getViewer().setShapeShow(13, true);
				jmolPdbViewer.getViewer().setShapeShow(14, true);
				jmolPdbViewer.getViewer().setShapeShow(15, true);
			}
		}
	}

	/**
	 * The JPreview is a container wich handles the specific JmolViewer in
	 * restricted mode used by cell microcosmos editor to preview different
	 * PDB-files. If wanted it is possible to switch to a standard Jmolviewer to
	 * use a viewer which runs not in restricted mode. Often used functions are
	 * accesible from different buttons on the JPreview panel.
	 * 
	 * @param workPanel
	 *            The used JWorkPanel contains informations about other objects
	 *            wich will be needed by JPreview.
	 */
	public JPreview(JWorkPanel workPanel) {

		// Load the lipid before viewer ist allocated, for lipid init!!!
		// open lipid
		this.workPanel = workPanel;
		// get the actual lipid-model
		MembranModel membranemodel = MembranModelSingleton.getInstance();
		lipid = membranemodel.getBiLayerType();
		lipid.startupLipid();

		pdbFiles = new Vector();

		this.setLayout(new BorderLayout());

		// Panel with MembraneViewer
		membraneJPanel = new JPanel();
		// Panel with ProteinLipidViewer
		jPanelJmol = new JPanel();
		jPanelJmol.setLayout(new BorderLayout());

		JPanel leftPanel = new JPanel();
		JPanel rightPanel = new JPanel();
		leftPanel.setBackground(Color.WHITE);
		rightPanel.setBackground(Color.WHITE);
		Image image = Toolkit.getDefaultToolkit().getImage(
				"images/lipide__1_1.jpg");

		PreviewLipidDrawer membraneLeft = new PreviewLipidDrawer(25, 100, 4,
				image);
		PreviewLipidDrawer membraneRight = new PreviewLipidDrawer(25, 100, 4,
				image);

		membraneRight.drawLipidImages();

		leftPanel.add(membraneLeft);
		rightPanel.add(membraneRight);

		// ////////////////////////////////////Create MembraneViewer
		// /////////////////////////
		jmolMemViewer = new JmolPdbPreview(membraneJPanel, lipid, false);
		jmolMemViewer.setLayout(new BorderLayout());
		jmolMemViewer.getViewer().setColorBackground(Color.WHITE);

		membraneJPanel.setBackground(Color.BLUE);
		membraneJPanel.setLayout(new BorderLayout());
		membraneJPanel.add(jmolMemViewer, BorderLayout.CENTER);

		// ////////////////////////////////////Create LipidMembraneViewer
		// /////////////////////////
		jmolPdbViewer = new JmolPdbPreview(jPanelJmol, lipid, true, this);
		jmolPdbViewer.setLayout(new BorderLayout());
		jmolPdbViewer.getViewer().setColorBackground(Color.WHITE);
		jmolPdbViewer.getViewer().setShowTransparentLipids(false);

		this.addComponentListener(new JPreviewResizedListener(this));
		jPanelJmol.add(jmolPdbViewer, BorderLayout.CENTER);
		jPanelJmol.setVisible(true);

		jPanelJmolButtons = new JPanel();
		jPanelJmolButtons.setLayout(new BorderLayout());

		// The Toggle-Lipid-Membrane-Button
		JButton lipidButton = new JButton("Toggle Lipidmembrane");
		lipidButton.addActionListener(new LipidButtonListener());
		jPanelJmolButtons.add(lipidButton, BorderLayout.CENTER);

		// The Toggle-Primary-Structure-Button
		JButton axesStructButton = new JButton("Toggle Axes");
		axesStructButton.addActionListener(new AxesButtonListener());
		jPanelJmolButtons.add(axesStructButton, BorderLayout.WEST);

		// The Toggle-Secondary-Structure-Button
		JButton secStructButton = new JButton("Toggle Secondary Structure");
		secStructButton.addActionListener(new SecStructButtonListener());
		jPanelJmolButtons.add(secStructButton, BorderLayout.EAST);
		jPanelJmolButtons.setVisible(true);

		// add the constructs to this JPreview panel
		this.setDoubleBuffered(true);
		this.add(jPanelJmol);
		this.add(jPanelJmolButtons, BorderLayout.SOUTH);
		this.repaint();

	}

	/**
	 * Computes the yTranslation and the RotationMatrix of the PDB. It gets the
	 * RotationMatrix from the JmolViewer and writes it to the protein.
	 * 
	 * @param givenProtein
	 *            The use pdb as Protein object.
	 */
	public void computePDBManipulation(Protein givenProtein) {
		// scalefactor proteine/lipid
		Matrix3f pdbRotationMatrix = new Matrix3f();
		jmolPdbViewer.getViewer().getRotation(pdbRotationMatrix);
		givenProtein.setRotationMatrix(pdbRotationMatrix);
		givenProtein.setYTranslation((-1)
				* (jmolPdbViewer.getViewer().getYTranslationNormalized()));

		// System.out.println("***COMPUTATION");
		// System.out.println("***Y-Translation  : "+protein.getYTranslation());
		// System.out.println("***RotationMatrix : "+pdbRotationMatrix.toString());
		// System.out.println("***Y-TransJmol    : "+jmolPdbViewer.getViewer().getYTranslationNormalized());
		// System.out.println("***ScaleFactorJmol: "+jmolPdbViewer.getViewer().getScalePixelsPerAngstrom());
	}

	/**
	 * Computes the yTranslation and the RotationMatrix of the PDB. It gets the
	 * RotationMatrix from the protein and writes it to the JmolViewer.
	 * 
	 * @param givenProtein
	 *            The use pdb as Protein object.
	 */
	public void fetchPDBManipulationFromProtein(Protein givenProtein) {
		Matrix3f pdbRotationMatrix = new Matrix3f();
		givenProtein.getRotationMatrix(pdbRotationMatrix);
		jmolPdbViewer.getViewer().setRotation(pdbRotationMatrix);
		jmolPdbViewer.getViewer().setYTranslationNormalized(
				(-1) * (givenProtein.getYTranslation4Jmol()));
		jmolPdbViewer.getViewer().scaleFitToScreen();

		// System.out.println("***FETCHING");
		// System.out.println("***Y-Translation : "+protein.getYTranslation());
		// System.out.println("***RotationMatrix:
		// "+pdbRotationMatrix.toString());
		// System.out.println("***Y-TransJmol :
		// "+jmolPdbViewer.getViewer().getYTranslationNormalized());
	}

	// repaint the whole preview
	public void updatePreview() {
		// workaround to repaint the pdb in order to correctly resize it
		this.setSize(this.getWidth() + 1, this.getHeight() + 1);
	}

	/**
	 * This method switches between the two viewer
	 * 
	 * @param viewerToShow
	 *            The viewer wich should be visualized
	 */
	public void switchViewerTo(JPanel viewerToShow) {
		this.removeAll();
		viewerToShow.setVisible(true);
		this.add(viewerToShow);
		this.repaint();
	}

	/**
	 * Reads a PDB file, wich is visualized in pdb-viewer
	 * 
	 * @param filepath
	 */
	public void showPdbFile(String filepath) {
		jmolPdbViewer.getViewer().openFile(filepath);

		String strError = jmolPdbViewer.getViewer().getOpenFileError();
		if (strError != null)
			System.out.println(strError);
	}

	/**
	 * Reads a PDB file, wich is visualized in the given viewer
	 * 
	 * @param filepath
	 *            Path of the file to display
	 * @param viewer
	 *            The viewer wich should visualize the given file
	 */
	public void showViewerPdbFile(String filepath, JmolPdbPreview viewer) {
		viewer.getViewer().openFile(filepath);

		String strError = viewer.getViewer().getOpenFileError();
		if (strError != null)
			System.out.println(strError);
	}

	/**
	 * Contains the pdb for the pdb-viewer in a vector
	 * 
	 * @param pdb
	 *            The vector, wich contains the pdb
	 */
	public void setPDB(Vector pdb) {
		if (pdb.size() == 1) {
			String toprint = "";
			DefaultMutableTreeNodeProtein node = (DefaultMutableTreeNodeProtein) pdb
					.elementAt(0);
			String name = node.getPDBName();
			protein = node.getProtein();
			// jmolPdbViewer.getViewer().openStringInline(protein.getCoordinatesAsString());
			showPdbFile(protein.getFilepath());
			jmolPdbViewer.getViewer().setScalePixelsPerAngstrom(
					jmolPdbViewer.getViewer().getRotationRadius() / 3);
			toprint = toprint + node + ": Prozentwerde-" + node.getPercent()
					+ "; File-" + node.getFile() + "; pdbName-" + name + "\n";
			// workPanel.updatePreview();
		}
	}

	/**
	 * This is only a test-method! Just visualize some Jmol informations and
	 * properties. Not longer used and this one will be deleted at the next
	 * release!
	 * 
	 */
	public void showInfo() {
		Dimension jmolPdbViewerDimension = new Dimension(200, 10);
		// jmolPdbViewer.setVisible(false);
		/**
		 * jmolPdbViewer.getViewer().pushHoldRepaint();
		 * jmolPdbViewer.setMaximumSize(jmolPdbViewerDimension);
		 * jmolPdbViewer.setMinimumSize(jmolPdbViewerDimension);
		 * jmolPdbViewer.setSize(jmolPdbViewerDimension);
		 * System.out.println("Zoom:"+jmolPdbViewer.getViewer().getZoomPercent());
		 * jmolPdbViewer.getViewer().setVectorScale(new Float(19));
		 * jmolPdbViewer.getViewer().setScreenDimension(jmolPdbViewerDimension);
		 * jmolPdbViewer.getViewer().setShowMeasurements(true);
		 * jmolPdbViewer.getViewer().setWireframeRotation(false);
		 * jmolPdbViewer.getViewer().setPerspectiveDepth(false);
		 * jmolPdbViewer.rectClip; jmolPdbViewer.getViewer().popHoldRepaint();
		 * jmolPdbViewer.getViewer().refresh(); jmolPdbViewer.setVisible(true);
		 * jmolPdbViewer.getViewer().homePosition();
		 * System.out.println("Height:"+jmolPdbViewer.currentSize); //?
		 * jmolPdbViewer.getViewer().setCenterSelected();
		 */

		jmolPdbViewer.getViewer().zoomBy(10);

		// jmolPdbViewer.getViewer().setColorBackground(Color.BLUE);
		if (jmolPdbViewer.getViewer().getShowAxes()) {
			jmolPdbViewer.getViewer().setShowAxes(false);
			jmolPdbViewer.getViewer().setShowBbcage(false);
		} else {
			jmolPdbViewer.getViewer().setShowAxes(true);
			jmolPdbViewer.getViewer().setShowBbcage(true);
		}
		// System.out.println("Size:" + jmolPdbViewer.getParent().getSize());
	}

	/**
	 * Gives an info, whether all transparents will be displayed or not.
	 * 
	 * @return Returns true if all transparent lipids are shown. If it returns
	 *         false only the ones on the left and on the right are shown
	 */
	public boolean getShowTransparentLipids() {
		return (jmolPdbViewer.getViewer().isShowTransparentLipids());
	}

	/**
	 * Sets whether all transparents lipids will be displayed or not.
	 * 
	 * @param showLipids
	 *            Set this to true and all transparent lipids will be displayed.
	 */
	public void setShowTransparentLipids(boolean showLipids) {
		jmolPdbViewer.getViewer().setShowTransparentLipids(showLipids);
		jmolPdbViewer.getViewer().refresh();
	}

	public void notifyFileLoaded(String fullPathName, String fileName,
			String modelName, Object clientFile, String errorMessage) {
		// TODO
	}

	public void setStatusMessage(String statusMessage) {
		// TODO
	}

	public void scriptEcho(String strEcho) {
		// TODO
	}

	public void scriptStatus(String strStatus) {
		// TODO
	}

	public void notifyScriptTermination(String statusMessage, int msWalltime) {
		// TODO
	}

	public void handlePopupMenu(int x, int y) {
		// TODO
	}

	public void notifyMeasurementsChanged() {
		// TODO
	}

	public void notifyFrameChanged(int frameNo) {
		// TODO
	}

	public void notifyAtomPicked(int atomIndex, String strInfo) {
		// TODO
	}

	public void showUrl(String url) {
		// TODO
	}

	public void showConsole(boolean showConsole) {
		// TODO
	}

	/**
	 * @return Returns the workPanel.
	 */
	public JWorkPanel getWorkPanel() {
		return workPanel;
	}

	/**
	 * @param workPanel
	 *            The workPanel to set.
	 */
	private void setWorkPanel(JWorkPanel workPanel) {
		this.workPanel = workPanel;
	}

	/**
	 * @return Returns the membraneJPanel.
	 */
	public JPanel getMembraneJPanel() {
		return membraneJPanel;
	}

	/**
	 * @param membraneJPanel
	 *            The membraneJPanel to set.
	 */
	public void setMembraneJPanel(JPanel membraneJPanel) {
		this.membraneJPanel = membraneJPanel;
	}

	/**
	 * @return Returns the jmolMemViewer.
	 */
	public JmolPdbPreview getJmolMemViewer() {
		return jmolMemViewer;
	}

	/**
	 * @param jmolMemViewer
	 *            The jmolMemViewer to set.
	 */
	public void setJmolMemViewer(JmolPdbPreview jmolMemViewer) {
		this.jmolMemViewer = jmolMemViewer;
	}
}
