package gui;

import gui.dragndrop.MyTreeModelListener;
import gui.dragndrop.TreeDragGestureListener;
import gui.dragndrop.TreeDropTargetListener;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragSource;
import java.awt.dnd.DropTarget;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;
import javax.vecmath.Matrix3f;

import membranModel.MembranModel;
import membranModel.MembranModelSingleton;
import pdbParser.PDBreader;
import protein.Protein;

public class Membran extends JPanel implements MouseListener, ActionListener {
	private JTree tree, pdbFiles;

	private JPopupMenu popupPDB, popupKomponente, popupMembran;

	private DefaultTreeModel model;

	private JMenuItem pdbDelete, pdbRename, pdbPreview, pdbShowProperties,
			pdbChangePercent;

	private JMenuItem membranRename, membranPreview, membranShowProperties,
			membranChangePercent;

	private JMenuItem kompDelete, kompRename, kompShowProperties;

	private DefaultMutableTreeNodeKatagory root;

	private JButton previewButton;

	int componenteNumb = 0;

	private Main main;

	public NewMembraneDialog nmd;

	TreeDropTargetListener tdtl;
/**
 * Creates the JPanel with the membrane model tree. 
 * @param main Main
 */
	public Membran(Main main) {
		pdbFiles = new JTree();
		this.main = main;
		BorderLayout layout = new BorderLayout();
		this.setLayout(layout);
		this.setPreferredSize(new Dimension(350, 200));

		root = new DefaultMutableTreeNodeKatagory(" ", 0);

		root.setIdeal_value(100);

		model = new DefaultTreeModel(root);
		tree = new JTree(model);
		tree.setRootVisible(false);
		tree.setBackground(MembranModelSingleton.getInstance()
				.getBackground_color());
		DefaultTreeCellRenderer linksRenderer = new DefaultTreeCellRenderer();
		linksRenderer.setBackgroundNonSelectionColor(MembranModelSingleton
				.getInstance().getBackground_color());
		linksRenderer.setBackgroundSelectionColor(MembranModelSingleton
				.getInstance().getSelection_color());
		// linksRenderer.setLeafIcon(new ImageIcon("middle.gif"));
		tree.setCellRenderer(linksRenderer);

		MyTreeModelListener myTreeModelListener = new MyTreeModelListener(tree);
		model.addTreeModelListener(myTreeModelListener);

		tree.addMouseListener(this);

		tree.setEditable(false);
		tree.getSelectionModel().setSelectionMode(
				TreeSelectionModel.SINGLE_TREE_SELECTION);
		tree.setShowsRootHandles(true);

		JScrollPane js = new JScrollPane(tree);

		this.add(js, BorderLayout.CENTER);

		// Komponentenmen
		popupKomponente = new JPopupMenu();

		kompDelete = new JMenuItem("Delete component");
		kompDelete.addActionListener(this);
		popupKomponente.add(kompDelete);

		kompRename = new JMenuItem("Rename component");
		popupKomponente.add(kompRename);
		kompRename.addActionListener(this);

		kompShowProperties = new JMenuItem("Show properties");
		kompShowProperties.addActionListener(this);
		popupKomponente.add(kompShowProperties);

		// PDBMen
		popupPDB = new JPopupMenu();

		pdbDelete = new JMenuItem("Delete protein");
		pdbDelete.addActionListener(this);
		popupPDB.add(pdbDelete);

		pdbRename = new JMenuItem("Rename protein");
		pdbRename.addActionListener(this);
		popupPDB.add(pdbRename);

		pdbPreview = new JMenuItem("Protein preview");
		pdbPreview.addActionListener(this);
		popupPDB.add(pdbPreview);

		pdbShowProperties = new JMenuItem("Show properties");
		pdbShowProperties.addActionListener(this);

		// pdbChangeProperties = new JMenuItem("Change Properties");
		// pdbChangeProperties.addActionListener(this);
		// popupPDB.add(pdbChangeProperties);

		pdbChangePercent = new JMenuItem("Change percent");
		pdbChangePercent.addActionListener(this);
		popupPDB.add(pdbChangePercent);

		// MembranMen
		popupMembran = new JPopupMenu();

		membranRename = new JMenuItem("Rename membrane");
		membranRename.addActionListener(this);
		popupMembran.add(membranRename);

		// membranPreview = new JMenuItem("Membrane preview");
		// membranPreview.addActionListener(this);
		// popupMembran.add(membranPreview);

		membranShowProperties = new JMenuItem("Show properties");
		membranShowProperties.addActionListener(this);
		popupMembran.add(membranShowProperties);
		/*
		 * membranChangeProperties = new JMenuItem("Change Properties");
		 * membranChangeProperties.addActionListener(this);
		 * popupMembran.add(membranChangeProperties);
		 */
		// membranChangePercent = new JMenuItem("Change percent");
		// membranChangePercent.addActionListener(this);
		// popupMembran.add(membranChangePercent);
		// quickStart();
	}
/**
 * Constructs a tree of a MembranModel. 
 * @param membraneModel MembranModel 
 */
	public void initTree(MembranModel membraneModel) {

		root.setUserObject(membraneModel.getName());
		int numb = root.getChildCount();
		for (int i = 0; i < numb; i++) {
			DefaultMutableTreeNode node = (DefaultMutableTreeNode) root
					.getChildAt(0);
			model.removeNodeFromParent(node);

		}
		model.nodeChanged(root);

		DefaultMutableTreeNode ionchannel = new DefaultMutableTreeNode(
				"ionchannel");
		model.insertNodeInto(ionchannel, root, root.getChildCount());
		model.nodeChanged(root);

		DefaultMutableTreeNode porin = new DefaultMutableTreeNode("porin");
		model.insertNodeInto(porin, root, root.getChildCount());
		model.nodeChanged(root);

		DefaultMutableTreeNode receptor = new DefaultMutableTreeNode("receptor");
		model.insertNodeInto(receptor, root, root.getChildCount());
		model.nodeChanged(root);

		Vector membranProteinVector = membraneModel.getAllProteins();
		System.out.println(membranProteinVector.size());
		for (int i = 0; i < membranProteinVector.size(); i++) {
			boolean added = false;

			Protein protein = (Protein) membranProteinVector.get(i);
			DefaultMutableTreeNodeProtein nodeName = new DefaultMutableTreeNodeProtein(
					protein.getNodeName(), protein.getPDBcode(), (int) (protein
							.getRatio() * 100));
			nodeName.setProtein(protein);
			nodeName.setFile(protein.getFilepath());
			nodeName.setPDBName(protein.getPDBcode());
			int nodeRatio = (int) (protein.getRatio() * 100);
			root.update_is_value(nodeRatio);
			nodeName.setPercent(nodeRatio);
			nodeName.setMin_percent(protein.get_xz_percentage());
			String componente = protein.getComponent();
			String subcomponent = protein.getSubcomponent();

			// System.out.println("Sub"+subcomponent.substring(0, 10));
			if (subcomponent.length() > 10
					&& subcomponent.substring(0, 10).equals("Component-")) {
				componenteNumb++;
			}
			DefaultMutableTreeNode kategory = null;
			DefaultMutableTreeNodeKatagory subkategory;
			if (componente.equals("ionchannel")) {
				kategory = (DefaultMutableTreeNode) root.getChildAt(0);
			} else if (componente.equals("receptor")) {
				kategory = (DefaultMutableTreeNode) root.getChildAt(2);
			} else if (componente.equals("porin")) {
				kategory = (DefaultMutableTreeNode) root.getChildAt(1);
			}
			// check if there exists a node subcomponent
			for (int j = 0; j < kategory.getChildCount(); j++) {
				subkategory = (DefaultMutableTreeNodeKatagory) kategory
						.getChildAt(j);
				if (subkategory.toString().equals((Object) subcomponent)) {
					model.insertNodeInto(nodeName, subkategory, subkategory
							.getChildCount());
					// subkategory.add(nodeName);
					subkategory
							.update_is_value((int) (protein.getRatio() * 100));
					added = true;

				}
			}
			// ...if there doesn't exitst a subcomponent node, create one...
			if (!added) {
				subkategory = new DefaultMutableTreeNodeKatagory(subcomponent,
						0); // DefaultMutableTreeNodeKategory
				model.insertNodeInto(subkategory, kategory, kategory
						.getChildCount());
				// kategory.add(subkategory);
				model.insertNodeInto(nodeName, subkategory, subkategory
						.getChildCount());
				// subkategory.add(nodeName);
				subkategory.update_is_value((int) (protein.getRatio() * 100));

			}

		}
		((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel", this)).main
				.setGenerate(true);
		tree.setRootVisible(true);
		tree.expandPath(new TreePath(root));
		((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel", this))
				.enableMyTabs(true);
		tdtl.setKomponentsCounter(componenteNumb);

	}

/**
 * Constructs a new empty tree (without proteins)
 *
 */
	public void newMembranModel() {

		componenteNumb = 0;
		nmd = new NewMembraneDialog(main);
		nmd.requestFocusInWindow();
		nmd.requestFocus();
		String newName = nmd.getName();
		if (nmd.getReturnValue()) {
			// root
			((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel", this))
					.enableMyTabs(false);
			((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel", this)).tp
					.setSelectedIndex(0);

			model.reload();
			root.setUserObject(newName);
			model.nodeChanged(root);
			root.setIdeal_value(100);
			root.setName(newName);
			int numb = root.getChildCount();
			for (int i = 0; i < numb; i++) {
				DefaultMutableTreeNode node = (DefaultMutableTreeNode) root
						.getChildAt(0);
				model.removeNodeFromParent(node);
			}
			// first children
			DefaultMutableTreeNode ionchannel = new DefaultMutableTreeNode(
					"ionchannel");
			model.insertNodeInto(ionchannel, root, root.getChildCount());
			model.nodeChanged(root);

			DefaultMutableTreeNode porin = new DefaultMutableTreeNode("porin");
			model.insertNodeInto(porin, root, root.getChildCount());
			model.nodeChanged(root);

			DefaultMutableTreeNode receptor = new DefaultMutableTreeNode(
					"receptor");
			model.insertNodeInto(receptor, root, root.getChildCount());
			model.nodeChanged(root);

			// Tree
			tree.setRootVisible(true);
			tree.expandPath(new TreePath(root));

			// Write first properties of Membrane
			MembranModel mm = MembranModelSingleton.getInstance();
			mm.removeAllComponents();
			mm.setDescription(nmd.getDescription());
			mm.setName(newName);
			mm.setMembraneTitle(newName);
			// mm.setPercent(nmd.getRatio());
			mm.setPercent(100);

			SimpleDateFormat fmt;
			fmt = new SimpleDateFormat();
			fmt.applyPattern("dd'-'MMM'-'yy");

			Calendar cal;
			cal = new GregorianCalendar();
			String date = fmt.format(cal.getTime());
			mm.setDate(date);
			mm.setCelltype(nmd.getCelltype());
			mm.setSpecies(nmd.getSpecies());
			System.out.println(nmd.getCelltype());
			((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel", this)).main
					.setGenerate(false);
			tdtl.setKomponentsCounter(componenteNumb);

		}

	}

/**
 * Describes the actions after right-clicking on a node in the membrane model tree.
 * @param e MouseEvent
 */

	public void mouseClicked(MouseEvent e) {
		int tab = ((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel",
				this)).getActiveTab();
		if ((!(e.isMetaDown()))) {// && (tab != 0) && (tab != 3)) { //
			// isMetaDown
			/*
			 * This function controls the settings of the tab-listener when the
			 * user clicks onto the right tree and opens the assigned tab. In
			 * addition it deactivates the PDBPreview if no PDB is selected in
			 * the tree in order to avoid errors.
			 */
			TreePath path = tree.getPathForLocation(e.getX(), e.getY());
			tree.setSelectionPath(path);
			if (path != null) {
				if (path.getPathCount() == 4) {
					DefaultMutableTreeNodeProtein tmp = (DefaultMutableTreeNodeProtein) tree
							.getLastSelectedPathComponent();
					((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel",
							this)).tp.setEnabledAt(1, true);
					if (tmp == null) {
						return;
					}
					updateProtein(tmp);
					((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel",
							this)).updatePreview();
				} else if (path.getPathCount() == 1) {
					if (((JWorkPanel) SwingUtilities.getAncestorNamed(
							"workpanel", this)).getActiveTab() == 4) {
						((JWorkPanel) SwingUtilities.getAncestorNamed(
								"workpanel", this)).properties
								.setPropertiesText(createPropertiesTextOfTree());
						((JWorkPanel) SwingUtilities.getAncestorNamed(
								"workpanel", this)).tp.setEnabledAt(1, false);
					} else {
						((JWorkPanel) SwingUtilities.getAncestorNamed(
								"workpanel", this)).tp.setSelectedIndex(4);
						((JWorkPanel) SwingUtilities.getAncestorNamed(
								"workpanel", this)).tp.setEnabledAt(1, false);
					}
				} else if (path.getPathCount() == 2) {
					((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel",
							this)).tp.setSelectedIndex(0);
					((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel",
							this)).tp.setEnabledAt(1, false);
				} else if (path.getPathCount() == 3) {
					if (((JWorkPanel) SwingUtilities.getAncestorNamed(
							"workpanel", this)).getActiveTab() == 4) {
						((JWorkPanel) SwingUtilities.getAncestorNamed(
								"workpanel", this)).properties
								.setPropertiesText(createPropertiesTextOfKomponte((DefaultMutableTreeNodeKatagory) tree
										.getLastSelectedPathComponent()));
						((JWorkPanel) SwingUtilities.getAncestorNamed(
								"workpanel", this)).tp.setEnabledAt(1, false);
					} else {
						((JWorkPanel) SwingUtilities.getAncestorNamed(
								"workpanel", this)).tp.setSelectedIndex(4);
						((JWorkPanel) SwingUtilities.getAncestorNamed(
								"workpanel", this)).tp.setEnabledAt(1, false);
					}
				}
			}
		}
	}

	private String createPropertiesTextOfKomponte(
			DefaultMutableTreeNodeKatagory selectedNode) {

		String toprint = selectedNode.getName() + "\n";
		for (int i = 0; i < selectedNode.getChildCount(); i++) {
			DefaultMutableTreeNodeProtein node = (DefaultMutableTreeNodeProtein) selectedNode
					.getChildAt(i);
			Protein p = node.getProtein();
			Vector descriptVector = p.getDescription();
			toprint = toprint + " Name: \t\t" + p.getNodeName() + "\n";
			toprint = toprint + " Description: \t\t"
					+ descriptVector.elementAt(0).toString() + "\n";
			for (int l = 1; l < descriptVector.size(); l++) {
				toprint = toprint + "\t\t" + descriptVector.elementAt(l) + "\n";
			}
			toprint = toprint + " Percent: \t\t"
					+ ((DefaultMutableTreeNodeProtein) node).getPercent()
					+ "%\n";
			Matrix3f tmpMatrix = new Matrix3f();
			p.getRotationMatrix(tmpMatrix);
			String matrix = tmpMatrix.toString();
			String[] tmp = matrix.split("\n");
			toprint = toprint + " Rotationmatrix: \t[" + tmp[1] + "\n";
			for (int e = 2; e < tmp.length; e++) {
				toprint = toprint + "\t\t " + tmp[e] + "\n";
			}
			toprint = toprint + " YTranslation: \t" + p.getYTranslation()
					+ "\n";
			toprint = toprint + " PDBCode\t\t" + p.getPDBcode() + "\n Date\t\t"
					+ p.getDate() + "\n\n";

		}
		return toprint;
	}

	private void updateProtein(DefaultMutableTreeNodeProtein tmp) {
		// ((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel",
		// this)).computePDBManipulation();
		((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel", this)).properties
				.setPropertiesText(createPropertiesText());
		// if(((JWorkPanel)
		// SwingUtilities.getAncestorNamed("workpanel",this)).getActiveTab()==1){
		Vector blubb = new Vector();
		blubb.addElement(tmp);
		if (((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel", this)).membranePreview != null) {
			((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel", this)).membranePreview
					.setPDB(blubb);
		}
		if (!((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel", this))
				.isPreview()) {
			((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel", this))
					.initPreview();
			((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel", this))
					.setPreview(true);
			((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel", this)).membranePreview
					.setPDB(blubb);
		}
		/*
		 * When the protein is updated, first the yTranslation and the
		 * rotationMatrix are fetched from the protein-object. Afterwards they
		 * are assigned to the Jmol- application. Finally, the view of the
		 * PDBPreview is updated.
		 */
		((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel", this))
				.fetchPDBManipulation();
		((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel", this))
				.computePDBManipulation();
		((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel", this))
				.updatePreview();
	}

	public void mouseEntered(MouseEvent e) {
	}

	public void mouseExited(MouseEvent e) {
	}

	public void mousePressed(MouseEvent e) {
	}
/**
 * Creates a popUpMenu after right-clicking and releasing. 
 * @param e MouseEvent
 */
	public void mouseReleased(MouseEvent e) {
		if ((e.isMetaDown())) { // && (tab == 0 || tab == 4))) {
			// =>right MouseClick
			if (!e.isPopupTrigger())
				return;
			TreePath path = tree.getPathForLocation(e.getX(), e.getY());
			tree.setSelectionPath(path);
			/*
			 * This function shows the popup-window in the tree. They are
			 * shifted to the left side of the arrow, because otherwise it may
			 * occur, that the PDBPreview is repainted using wrong values and
			 * the view afterwards is buggy.
			 */
			if (path != null) {
				switch (path.getPathCount()) {
				case 1:
					popupMembran.show(tree, (e.getX() - 132), e.getY());
					((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel",
							this)).tp.setSelectedIndex(0);
					((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel",
							this)).tp.setEnabledAt(1, false);
					break;
				case 2:
					((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel",
							this)).tp.setSelectedIndex(0);
					((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel",
							this)).tp.setEnabledAt(1, false);
					break;
				case 3:
					popupKomponente.show(tree, (e.getX() - 136), e.getY());
					((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel",
							this)).tp.setSelectedIndex(0);
					((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel",
							this)).tp.setEnabledAt(1, false);
					break;
				case 4:
					popupPDB.show(tree, (e.getX() - 112), e.getY());
					break;
				}
			}
		}
	}


/**
 * Listes the actions to perform after selecting items in the popUpMenu.
 * @param e ActionEvent
 */
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == pdbDelete || e.getSource() == (kompDelete)) {
			removeNode(e.getSource());
			if (this.getProteins().isEmpty()) {
				((JWorkPanel) SwingUtilities
						.getAncestorNamed("workpanel", this)).tp.setEnabledAt(
						1, false);
				((JWorkPanel) SwingUtilities
						.getAncestorNamed("workpanel", this)).tp.setEnabledAt(
						2, false);
				((JWorkPanel) SwingUtilities
						.getAncestorNamed("workpanel", this)).tp.setEnabledAt(
						3, false);
				((JWorkPanel) SwingUtilities
						.getAncestorNamed("workpanel", this)).tp.setEnabledAt(
						4, false);
				int index = ((JWorkPanel) SwingUtilities.getAncestorNamed(
						"workpanel", this)).tp.getSelectedIndex();
				if (index == 1 || index == 2 || index == 3 || index == 4) {
					((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel",
							this)).tp.setSelectedIndex(0);
				}
			} else {
				// hier: das letzte Element des Vektors im Preview anzeigen und
				// links aktivieren:
				// ((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel",
				// this)).setPreviewTab((Vector)this.getProteins().lastElement());
				//
				((JWorkPanel) SwingUtilities
						.getAncestorNamed("workpanel", this)).tp
						.setSelectedIndex(0);
				((JWorkPanel) SwingUtilities
						.getAncestorNamed("workpanel", this)).tp.setEnabledAt(
						1, false);

			}
			return;
		}
		if (e.getSource() == kompRename || e.getSource() == membranRename
				|| e.getSource() == pdbRename) {
			renameNode(e.getSource());
			return;
		}

		if (e.getSource() == pdbPreview) {
			Vector blubb = new Vector();
			blubb.add(tree.getLastSelectedPathComponent());
			((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel", this))
					.setPreviewTab(blubb);
			/*
			 * First the yTranslation and the rotationMatrix are fetched from
			 * the protein-object, then they are assigned to the
			 * Jmol-application.
			 */
			((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel", this))
					.fetchPDBManipulation();
			((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel", this))
					.computePDBManipulation();
			return;
		}

		if (e.getSource() == pdbShowProperties) {
			// ((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel",
			// this)).computePDBManipulation();
			String toprint = createPropertiesText();
			((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel", this))
					.setPropertiesTab(toprint);
			return;
		}

		if (e.getSource() == pdbChangePercent) {
			DefaultMutableTreeNodeProtein selected = (DefaultMutableTreeNodeProtein) tree
					.getLastSelectedPathComponent();
			DefaultMutableTreeNodeKatagory parent = (DefaultMutableTreeNodeKatagory) selected
					.getParent();
			ChangePercent newPercent = new ChangePercent(main, selected
					.toString(), "Protein", (root.getIdeal_value() - root
					.getIs_value())
					+ selected.getPercent(), selected.getMin_percent(),
					selected.getPercent());
			if (!newPercent.getChangen()) {
				return;
			}
			root.update_is_value(Integer.parseInt(newPercent.getRatio())
					- selected.getPercent());
			parent.update_is_value(Integer.parseInt(newPercent.getRatio())
					- selected.getPercent());
			selected.setPercent(Integer.parseInt(newPercent.getRatio()));
			int nodeRatioValue = (Integer.parseInt(newPercent.getRatio()));
			float tmp = ((float) nodeRatioValue / 100);
			selected.getProtein().setRatio(tmp);
			MembranModel mm = MembranModelSingleton.getInstance();
			// Protein tmpProtein = mm.getProtein(selected.getPDBName());
			// tmpProtein.setRatio(tmp);

			mm.getProtein(selected.getPDBName() + "_" + parent.getName())
					.setRatio(tmp);
			mm.getProtein(selected.getPDBName() + "_" + parent.getName())
					.setQuantity(nodeRatioValue);
			((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel", this)).properties
					.setPropertiesText(createPropertiesText());
			return;
		}

		if (e.getSource() == membranPreview) {
			((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel", this))
					.setPreviewTab(treeToVector());
			((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel", this)).tp
					.setEnabledAt(1, true);
			return;
		}

		if (e.getSource() == membranShowProperties) {
			String toprint = createPropertiesTextOfTree();
			((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel", this))
					.setPropertiesTab(toprint);
			return;
		}

		if (e.getSource() == membranChangePercent) {
			ChangePercent newPercent = new ChangePercent(main, root.toString(),
					"Membrane", 100, (root.getIs_value()), root
							.getIdeal_value());
			root.setIdeal_value(Integer.parseInt(newPercent.getRatio()));
			MembranModelSingleton.getInstance().setPercent(
					Integer.parseInt(newPercent.getRatio()));
			float tmp = Float.parseFloat("0." + newPercent.getRatio());
			if (((JWorkPanel) SwingUtilities
					.getAncestorNamed("workpanel", this)).getActiveTab() == 4) {
				((JWorkPanel) SwingUtilities
						.getAncestorNamed("workpanel", this)).properties
						.setPropertiesText(createPropertiesTextOfTree());
			}
			return;
		}

		if (e.getSource() == kompShowProperties) {
			DefaultMutableTreeNodeKatagory selectedNode = (DefaultMutableTreeNodeKatagory) tree
					.getLastSelectedPathComponent();
			String toprint = createPropertiesTextOfKomponte(selectedNode);
			((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel", this))
					.setPropertiesTab(toprint);
			return;
		}
		if (e.getSource() == previewButton) {
			DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode) tree
					.getLastSelectedPathComponent();
			if (selectedNode == null) {
				return;
			}
			switch (selectedNode.getPath().length) {
			// Root
			case 1:
				((JWorkPanel) SwingUtilities
						.getAncestorNamed("workpanel", this))
						.setPreviewTab(treeToVector());
				break;
			// Category
			case 2:
				break;
			// Component
			case 3:
				Vector blubb = new Vector();
				for (int i = 0; i < selectedNode.getChildCount(); i++) {
					blubb.add(selectedNode.getChildAt(i));
				}
				((JWorkPanel) SwingUtilities
						.getAncestorNamed("workpanel", this))
						.setPreviewTab(blubb);
				break;
			// pdb
			case 4:
				Vector blubb2 = new Vector();
				blubb2.add(tree.getLastSelectedPathComponent());
				((JWorkPanel) SwingUtilities
						.getAncestorNamed("workpanel", this))
						.setPreviewTab(blubb2);
			}
		} else {
			((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel", this)).tp
					.setSelectedIndex(4);
			((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel", this)).tp
					.setEnabledAt(1, false);
		}
	}

	/**
	 * Returns the text for the properties panel for a node.
	 * @return text to print 
	 */
	public String createPropertiesText() {
		String toprint = "";

		DefaultMutableTreeNode node = (DefaultMutableTreeNode) tree
				.getLastSelectedPathComponent();
		if (node == null) {
			return "";
		}
		switch (node.getPath().length) {

		case 1:
			toprint = createPropertiesTextOfTree();
			break;
		case 3:
			toprint = createPropertiesTextOfKomponte((DefaultMutableTreeNodeKatagory) node);
			break;
		case 4:
			Protein p = ((DefaultMutableTreeNodeProtein) node).getProtein();
			Vector descriptVector = p.getDescription();
			toprint = " Name: \t\t" + p.getNodeName() + "\n";
			if (descriptVector.size() != 0) {
				toprint = toprint + " Description: \t\t"
						+ descriptVector.elementAt(0).toString() + "\n";
			}
			for (int l = 1; l < descriptVector.size(); l++) {
				toprint = toprint + "\t\t" + descriptVector.elementAt(l) + "\n";
			}
			toprint = toprint + " Percent: \t\t"
					+ ((DefaultMutableTreeNodeProtein) node).getPercent()
					+ "%\n";
			Matrix3f tmpMatrix = new Matrix3f();
			p.getRotationMatrix(tmpMatrix);
			String matrix = tmpMatrix.toString();
			String[] tmp = matrix.split("\n");
			toprint = toprint + " Rotationmatrix: \t[" + tmp[1] + "\n";
			for (int i = 2; i < tmp.length; i++) {
				toprint = toprint + "\t\t " + tmp[i] + "\n";
			}
			toprint = toprint + " YTranslation: \t" + p.getYTranslation()
					+ "\n";
			toprint = toprint + " PDBCode\t\t" + p.getPDBcode() + "\n Date\t\t"
					+ p.getDate();
		}
		return toprint;
	}

	private String createPropertiesTextOfTree() {
		String toprint = "";
		DefaultMutableTreeNodeKatagory root = (DefaultMutableTreeNodeKatagory) model
				.getRoot();
		MembranModel membranModel = MembranModelSingleton.getInstance();
		toprint = "Membranename: " + root.getName() + "\n";
		// shows the description of the membrane
		if (membranModel.getDescription().length() > 0) {
			toprint = toprint + "\nDescription: "
					+ membranModel.getDescription() + "\n";
		}
		toprint = toprint + "\n";
		// iteration ber die drei Kategorien
		for (int i = 0; i < 3; i++) {
			DefaultMutableTreeNode kategorie = (DefaultMutableTreeNode) root
					.getChildAt(i);
			// iteration ber komponenten jeder Kategorie
			for (int j = 0; j < kategorie.getChildCount(); j++) {
				DefaultMutableTreeNodeKatagory komponente = (DefaultMutableTreeNodeKatagory) kategorie
						.getChildAt(j);
				toprint = toprint + "\n" + komponente.getName() + "\n";
				// iteration ber pdb files
				for (int k = 0; k < komponente.getChildCount(); k++) {
					DefaultMutableTreeNodeProtein node = (DefaultMutableTreeNodeProtein) komponente
							.getChildAt(k);
					Protein p = node.getProtein();
					Vector descriptVector = p.getDescription();
					toprint = toprint + " Name: \t\t" + p.getNodeName() + "\n";
					toprint = toprint + " Description: \t\t"
							+ descriptVector.elementAt(0).toString() + "\n";
					for (int l = 1; l < descriptVector.size(); l++) {
						toprint = toprint + "\t\t"
								+ descriptVector.elementAt(l) + "\n";
					}
					toprint = toprint
							+ " Percent: \t\t"
							+ ((DefaultMutableTreeNodeProtein) node)
									.getPercent() + "%\n";
					Matrix3f tmpMatrix = new Matrix3f();
					p.getRotationMatrix(tmpMatrix);
					String matrix = tmpMatrix.toString();
					String[] tmp = matrix.split("\n");
					toprint = toprint + " Rotationmatrix: \t[" + tmp[1] + "\n";
					for (int e = 2; e < tmp.length; e++) {
						toprint = toprint + "\t\t " + tmp[e] + "\n";
					}
					toprint = toprint + " YTranslation: \t"
							+ p.getYTranslation() + "\n";
					toprint = toprint + " PDBCode\t\t" + p.getPDBcode()
							+ "\n Date\t\t" + p.getDate() + "\n\n";

				}
			}

		}
		return toprint;
	}

	/*
	 * change the Percent value for the kategory node
	 */
	private void changePercent() {
		DefaultMutableTreeNodeKatagory selected = (DefaultMutableTreeNodeKatagory) tree
				.getLastSelectedPathComponent();
		if (selected == null)
			return;

		ChangePercent newPercent = new ChangePercent(main, selected.toString(),
				"Component ",
				(root.getIdeal_value() - root.getIs_value() + selected
						.getIdeal_value()), 0, selected.getIs_value());

		root.update_is_value(Integer.parseInt(newPercent.getRatio())
				- selected.getIdeal_value());
		return;

	}

	private Vector treeToVector() {
		Vector blubb = new Vector();
		DefaultMutableTreeNode root = (DefaultMutableTreeNode) model.getRoot();
		// iteration ber die drei Kategorien
		for (int i = 0; i < 3; i++) {
			DefaultMutableTreeNode kategorie = (DefaultMutableTreeNode) root
					.getChildAt(i);
			// iteration ber komponenten jeder Kategorie
			for (int j = 0; j < kategorie.getChildCount(); j++) {
				DefaultMutableTreeNode komponente = (DefaultMutableTreeNode) kategorie
						.getChildAt(j);
				// iteration ber pdb files
				for (int k = 0; k < komponente.getChildCount(); k++) {
					blubb.add(komponente.getChildAt(k));
				}
			}
		}
		return blubb;
	}

	private void renameNode(Object object) {
		DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode) tree
				.getLastSelectedPathComponent();
		if (selectedNode == null)
			return;

		String newName = (String) JOptionPane
				.showInputDialog("Enter a new name for " + selectedNode);
		if (newName == null) {
			return;
		}
		if (newName.equals(""))
			return;
		if (object == pdbRename) {
			((DefaultMutableTreeNodeProtein) selectedNode).setNodeName(newName);
			((DefaultMutableTreeNodeProtein) selectedNode).getProtein()
					.setNodeName(newName);
		} else if (object == kompRename) {
			((DefaultMutableTreeNodeKatagory) selectedNode).setName(newName);
			for (int i = 0; i < selectedNode.getChildCount(); i++) {
				Protein protein = ((DefaultMutableTreeNodeProtein) selectedNode
						.getChildAt(i)).getProtein();
				MembranModel membranemodel = MembranModelSingleton
						.getInstance();
				String oldName = protein.getPDBcode() + "_"
						+ protein.getSubcomponent();
				membranemodel.removeComponent(oldName);
				protein.setSubcomponent(newName);
				membranemodel.addProtein(protein);
				((JWorkPanel) SwingUtilities
						.getAncestorNamed("workpanel", this)).msg_panel.proteinMatrix
						.changeKey(oldName,
								(protein.getPDBcode() + "_" + protein
										.getSubcomponent()));
			}
		} else if (object == membranRename) {
			MembranModel membranemodel = MembranModelSingleton.getInstance();
			membranemodel.setName(newName);
		}


		selectedNode.setUserObject(newName);
		model.nodeChanged(selectedNode);

	}

	private void removeNode(Object object) {
		DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode) tree
				.getLastSelectedPathComponent();
		MembranModel membranModel = MembranModelSingleton.getInstance();
		if (selectedNode == null)
			return;
		MutableTreeNode parent = (MutableTreeNode) (selectedNode.getParent());

		if (parent != null) {
			if (object == pdbDelete) {
				((DefaultMutableTreeNodeKatagory) parent)
						.update_is_value(((DefaultMutableTreeNodeProtein) selectedNode)
								.getPercent()
								* (-1));
				root
						.update_is_value(((DefaultMutableTreeNodeProtein) selectedNode)
								.getPercent()
								* (-1));
				membranModel
						.removeComponent(((DefaultMutableTreeNodeProtein) selectedNode)
								.getPDBName()
								+ "_" + parent.toString());
				Protein protein = ((DefaultMutableTreeNodeProtein)selectedNode).getProtein();		
				((JWorkPanel) SwingUtilities
						.getAncestorNamed("workpanel", this)).msg_panel.proteinMatrix
						.deleteKey(protein.getPDBcode() + "_" + protein
										.getSubcomponent());
			}
			if (object == kompDelete) {
				for (int i = 0; i < selectedNode.getChildCount(); i++) {
					// System.out.println("Deleted:
					// "+(((DefaultMutableTreeNodeProtein)
					// selectedNode.getChildAt(i)).getPDBName()+"_"+selectedNode.getUserObject()));
					membranModel
							.removeComponent(((DefaultMutableTreeNodeProtein) selectedNode
									.getChildAt(i)).getPDBName()
									+ "_" + selectedNode.getUserObject());
				}
				root
						.update_is_value(((DefaultMutableTreeNodeKatagory) selectedNode)
								.getIs_value()
								* (-1));

			}
			model.removeNodeFromParent(selectedNode);

		}
	}

	/**
	 * returns proteins of a node in a vector
	 */
	public Vector getProteins() {
		Vector allProteins = new Vector();
		DefaultMutableTreeNode root = (DefaultMutableTreeNode) model.getRoot();
		// iteration ber die drei Kategorien
		for (int i = 0; i < 3; i++) {
			DefaultMutableTreeNode kategorie = (DefaultMutableTreeNode) root
					.getChildAt(i);
			// iteration ber komponenten jeder Kategorie
			for (int j = 0; j < kategorie.getChildCount(); j++) {
				DefaultMutableTreeNodeKatagory komponente = (DefaultMutableTreeNodeKatagory) kategorie
						.getChildAt(j);
				// iteration ber pdb files
				for (int k = 0; k < komponente.getChildCount(); k++) {
					allProteins.add(((DefaultMutableTreeNodeProtein) komponente
							.getChildAt(k)).getProtein());
				}
			}
		}

		return allProteins;

	}

	/**
	 * Returns last selected protein node in the tree.
	 * @return Returns selected node. 
	 */
	public DefaultMutableTreeNodeProtein getSelectedProteinNode() {
		DefaultMutableTreeNodeProtein selectedNode = (DefaultMutableTreeNodeProtein) tree
				.getLastSelectedPathComponent();
		return selectedNode;
	}
	
	/**
	 * Sets all the listeners necessary for Drag and Drog for the PDB tree and membrane model tree.
	 * @param input JTree to set
	 * @param main Main
	 */
	public void setJTreePDB(JTree input, Main main) {
		pdbFiles = input;
		DragSource dragSource = DragSource.getDefaultDragSource();
		TreeDragGestureListener tdgl = new TreeDragGestureListener();
		tdtl = new TreeDropTargetListener(tree, main, pdbFiles, componenteNumb);
		dragSource.createDefaultDragGestureRecognizer(pdbFiles,
				DnDConstants.ACTION_COPY_OR_MOVE, tdgl);
		DropTarget dropTarget = new DropTarget(tree, tdtl);
	}
	
	/**
	 * Returns the membrane model tree.
	 * @return Returns the tree.
	 */
	public JTree getTree() {
		return tree;
	}
}