/*
 * CVS-informations
 * 
 * $Author: sebastian $
 * $Date: 2006/06/13 15:27:14 $
 * $Revision: 1.33 $
 * 
 */
package membranModel;
import java.awt.Color;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;

import javax.vecmath.Matrix3f;

import pdbParser.PDBreader;
import protein.Protein;
/**
 * 
 * This class implements a membrane model
 *
 */
public class MembranModel {

	PDBreader pdbReader = new PDBreader();
	
	String vrml ="";
	private Hashtable hash = new Hashtable();
	private Protein biLayerType;
	private String description="uncommented";
	private String name="undefined";
	private String[] species;
	private String celltype;
	private String date;
	private String title ="";
	private int percent;
	private double biLayerSpace = 0;
	private double xz_area = 0;
	
	private int colorCounter = 0;
	private Color background_color = new Color(240, 240, 255);
	private Color selection_color = new Color(222, 222, 255);
	private Color[] colors = {new Color(25,25,112),							  
							  new Color(110,123,139),
							  new Color(139,69,19),
							  new Color(110,139,61),
							  new Color(139,139,122),
							  new Color(65,105,225),
							  new Color(139,134,78),
							  new Color(139,90,0),
							  new Color(218,165,32),
							  new Color(1,1,1)};
	
	/**
	 * 
	 * @param PDBCode
	 * @return Protein
	 */
	public Protein getProtein(String PDBCode) {
		return (Protein)hash.get(PDBCode);
	}

/**
 * This function adds a protein to a subcomponent of the mebrane model
 * @param PDBCode
 * @param sub_componente
 * @throws IOException
 */
	public void addComponent(String PDBCode, String sub_componente) throws IOException {
		hash.put(PDBCode+"_"+sub_componente,pdbReader.getProtein(PDBCode,"Files/"));
		if (++colorCounter >= colors.length) { colorCounter -= colors.length; };
		((Protein)hash.get(PDBCode+"_"+sub_componente)).setProteinColor(colors[colorCounter]);
		
	}
	
	/**
	 * This function adds one protein to the membrane model
	 * @param protein
	 */
	public void addProtein(Protein protein) {
		hash.put(protein.getPDBcode()+"_"+protein.getSubcomponent(),protein);
	}

/**
 * This function adds proteins to the membrane model 
 * @param PDBCodes
 */
	public void addComponents(Vector PDBCodes) {
		for(Enumeration el=PDBCodes.elements();el.hasMoreElements();){
			Protein current_protein=(Protein)el.nextElement();
			hash.put(current_protein.getECN(),current_protein);
		}
		 
	}
	
	

/**
 * This function removes a given protein
 * @param PDBCode
 */
	public void removeComponent(String PDBCode) {
		hash.remove(PDBCode);
		
	}

/**
 * this function removes all proteins with given PDBcode 
 * @param PDBCodes
 */
	public void removeComponents(Set PDBCodes) {
		Iterator it = PDBCodes.iterator();
		while(it.hasNext()){
			hash.remove(it.next().toString());
		}
		
	}
	/**
	 * this function removes all proteins that are in the membrane model
	 *
	 */
	public void removeAllComponents(){
		hash = new Hashtable();
	}

/**
 * sets the dirscription of the membrane model
 * @param Description
 */
	public void setDescription(String Description) {
		this.description=Description;
		
	}

/**
 * 
 * @return description of the membrane model
 */
	public String getDescription() {
		// TODO Auto-generated method stub
		return description;
	}

	/**
	 * @param name The name of the new BilayerType
	 * Sets a new lipid a BiLayerType
	 */
	public void setBiLayerType(String name) throws IOException  {
		biLayerType=pdbReader.getProtein(name,"Files/");
		
		if(name.equals("pc2_exp")){	
			//System.out.println("Matrix vom bilayer gesetzt!");
			biLayerType.setRotationMatrix(new Matrix3f((float)0.1197, (float)-0.594, (float)-0.7955,(float)-0.977, (float)-0.21, (float)0.011,(float)-0.175, (float)0.776, (float)-0.606));
		}
		biLayerType.completeScaling();
		setxz_area();
	}
/**
 * 
 * @return all proteins in the membrane model
 */
	public Vector getAllProteins() {		
		Vector vec = new Vector();
		Enumeration e =hash.keys();
		while(e.hasMoreElements()){
			vec.add(hash.get((String)e.nextElement()));
		}		
		return vec;
	}

/**
 * 
 * @return the names of the proteins in the membrane model
 */
	public Set getProteinNames() {
		// TODO Auto-generated method stub
		return hash.keySet();
	}
	
	
	/**
	 * @return Returns the celltype.
	 */
	public String getCelltype() {
		return celltype;
	}
	
	/**
	 * @param celltype The celltype to set.
	 */
	public void setCelltype(String celltype) {
		this.celltype = celltype;
	}
	
/**
 * 
 * @return the membrane title
 */
	public String getMembraneTitle() {
		// TODO Auto-generated method stub
		return title;
	}

/**
 * sets the membrane title
 * @param title
 */
	public void setMembraneTitle(String title) {
		this.title=title;
		
	}



	/**
	 * @param date The date to set.
	 */
	public void setDate(String date) {
		this.date = date;
	}


	/**
	 * @return Returns the name.
	 */
	public String getName() {
		return name;
	}


	/**
	 * @param name The name to set.
	 */
	public void setName(String name) {
		this.name = name;
	}


	/**
	 * @return Returns the species.
	 */
	public String[] getSpecies() {
		if(species == null){
			String[] tmp = {""};
			return tmp;
		}
		else{
			return species;
		}
	}


	/**
	 * @param species The species to set.
	 */
	public void setSpecies(String[] species) {
		this.species = species;
	}
	

	/**
	 * @return Returns the date.
	 */
	public String getDate() {
		return date;
	}

	/**
	 * 
	 * @return the amount of the proteins in the membrane model
	 */
	public int countProteins() {
		
		return hash.size();
	}


	/**
	 * @return Returns the percent.
	 */
	public int getPercent() {
		return percent;
	}


	/**
	 * @param percent The percent to set.
	 */
	public void setPercent(int percent) {
		this.percent = percent;
	}


	/**
	 * @return Returns the biLayerType.
	 */
	public Protein getBiLayerType() {
		return biLayerType;
	}


	/**
	 * @return Returns the colors.
	 */
	public Color[] getColors() {
		return colors;
	}
	/**
	 * 
	 * @param i
	 * @return the color
	 */
	public Color getColor(int i) {
		while (i > getColors().length) {
			i -= getColors().length;
		}
		return colors[i];
	}


	/**
	 * @return Returns the hash.
	 */
	public Hashtable getHash() {
		return hash;
	}


	/**
	 * @param hash The hash to set.
	 */
	public void setHash(Hashtable hash) {
		this.hash = hash;
	}
	
	/**
	 * Does the completeScaling() function for any Protein the membrane consists of
	 * @see completeScaling in ProteinClass
	 */
	public void complete_scaling_proteins(){
		Vector vector = getAllProteins();
		Iterator it = vector.iterator();
		
		while(it.hasNext()){
			Protein protein = (Protein)it.next();
			protein.completeScaling();
		}
	}


	/**
	 * @return Returns the background_color.
	 */
	public Color getBackground_color() {
		return background_color;
	}


	/**
	 * @param background_color The background_color to set.
	 */
	public void setBackground_color(Color background_color) {
		this.background_color = background_color;
	}


	/**
	 * @return Returns the selection_color.
	 */
	public Color getSelection_color() {
		return selection_color;
	}


	/**
	 * @param selection_color The selection_color to set.
	 */
	public void setSelection_color(Color selection_color) {
		this.selection_color = selection_color;
	}


	/**
	 * @return Returns the biLayerSpace.
	 */
	public double getBiLayerSpace() {
		return biLayerSpace;
	}


	/**
	 * @param biLayerSpace The biLayerSpace to set.
	 */
	public void setBiLayerSpace(double biLayerSpace) {
		this.biLayerSpace = biLayerSpace;
	}


	/**
	 * @return Returns the value of the xz_area, the space of the membrane in x- and z-dimension in angstrom^2. 
	 */
	public double getXz_area() {
		return xz_area;
	}


	/**
	 * @param xz_area The xz_area to set.
	 */
	public void setxz_area(double xz_area) {
		this.xz_area = xz_area;
	}
	
	/**
	 * Computes the value of the xz_area of the membrane, the area of the membrane in x- and z-dimension in angstrom^2. 
	 */
	public void setxz_area() {
		xz_area = 60*biLayerType.getSize()[0] *40*biLayerType.getSize()[2];
	}


	/**
	 * @return Returns the vrml.
	 */
	public String getVRMLFile() {
		return vrml;
	}


	/**
	 * @param vrml The vrml to set.
	 */
	public void setVRMLFile(String vrml) {
		this.vrml = vrml;
	}
	
	
}

